@@ -4,10 +4,16 @@ Note
 ----
 This is an internal API not covered by versioning policy.
 """
+
+from __future__ import annotations
+
+import logging
 import re
 import traceback
 import typing as t
-from collections.abc import Iterable, Mapping, Sequence
+from collections.abc import Callable, Iterable, Mapping, Sequence
+
+logger = logging.getLogger(__name__)
 
 if t.TYPE_CHECKING:
 
@@ -16,8 +22,8 @@ if t.TYPE_CHECKING:
 
         def __call__(
             self,
-            data: t.Union[str, t.List[str], "Mapping[str, str]"],
-            rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+            data: str | list[str] | Mapping[str, str],
+            rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
         ) -> bool:
             """Return callback for :class:`QueryList` filtering operators."""
             ...
@@ -29,17 +35,17 @@ no_arg = object()
 
 
 class MultipleObjectsReturned(Exception):
-    """The requested object does not exist."""
-
-
-class ObjectDoesNotExist(Exception):
     """The query returned multiple objects when only one was expected."""
 
 
+class ObjectDoesNotExist(Exception):
+    """The requested object does not exist."""
+
+
 def keygetter(
-    obj: "Mapping[str, t.Any]",
+    obj: Mapping[str, t.Any],
     path: str,
-) -> t.Union[None, t.Any, str, t.List[str], "Mapping[str, str]"]:
+) -> None | t.Any | str | list[str] | Mapping[str, str]:
     """Fetch values in objects and keys, supported nested data.
 
     **With dictionaries**:
@@ -101,17 +107,17 @@ def keygetter(
 
     except Exception as e:
         traceback.print_stack()
-        print(f"Above error was {e}")
+        logger.debug(f"The above error was {e}")
         return None
 
     return dct
 
 
 def parse_lookup(
-    obj: "Mapping[str, t.Any]",
+    obj: Mapping[str, t.Any],
     path: str,
     lookup: str,
-) -> t.Optional[t.Any]:
+) -> t.Any | None:
     """Check if field lookup key, e.g. "my__path__contains" has comparator, return val.
 
     If comparator not used or value not found, return None.
@@ -140,21 +146,22 @@ def parse_lookup(
             field_name = path.rsplit(lookup)[0]
             if field_name is not None:
                 return keygetter(obj, field_name)
-    except Exception:
+    except Exception as e:
         traceback.print_stack()
+        logger.debug(f"The above error was {e}")
     return None
 
 
 def lookup_exact(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     return rhs == data
 
 
 def lookup_iexact(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if not isinstance(rhs, str) or not isinstance(data, str):
         return False
@@ -163,8 +170,8 @@ def lookup_iexact(
 
 
 def lookup_contains(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if not isinstance(rhs, str) or not isinstance(data, (str, Mapping, list)):
         return False
@@ -173,8 +180,8 @@ def lookup_contains(
 
 
 def lookup_icontains(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if not isinstance(rhs, str) or not isinstance(data, (str, Mapping, list)):
         return False
@@ -188,8 +195,8 @@ def lookup_icontains(
 
 
 def lookup_startswith(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if not isinstance(rhs, str) or not isinstance(data, str):
         return False
@@ -198,8 +205,8 @@ def lookup_startswith(
 
 
 def lookup_istartswith(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if not isinstance(rhs, str) or not isinstance(data, str):
         return False
@@ -208,8 +215,8 @@ def lookup_istartswith(
 
 
 def lookup_endswith(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if not isinstance(rhs, str) or not isinstance(data, str):
         return False
@@ -218,8 +225,8 @@ def lookup_endswith(
 
 
 def lookup_iendswith(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if not isinstance(rhs, str) or not isinstance(data, str):
         return False
@@ -227,8 +234,8 @@ def lookup_iendswith(
 
 
 def lookup_in(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if isinstance(rhs, list):
         return data in rhs
@@ -249,8 +256,8 @@ def lookup_in(
 
 
 def lookup_nin(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if isinstance(rhs, list):
         return data not in rhs
@@ -271,8 +278,8 @@ def lookup_nin(
 
 
 def lookup_regex(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if isinstance(data, (str, bytes, re.Pattern)) and isinstance(rhs, (str, bytes)):
         return bool(re.search(rhs, data))
@@ -280,15 +287,15 @@ def lookup_regex(
 
 
 def lookup_iregex(
-    data: t.Union[str, t.List[str], "Mapping[str, str]"],
-    rhs: t.Union[str, t.List[str], "Mapping[str, str]", "re.Pattern[str]"],
+    data: str | list[str] | Mapping[str, str],
+    rhs: str | list[str] | Mapping[str, str] | re.Pattern[str],
 ) -> bool:
     if isinstance(data, (str, bytes, re.Pattern)) and isinstance(rhs, (str, bytes)):
         return bool(re.search(rhs, data, re.IGNORECASE))
     return False
 
 
-LOOKUP_NAME_MAP: 'Mapping[str, "LookupProtocol"]' = {
+LOOKUP_NAME_MAP: Mapping[str, LookupProtocol] = {
     "eq": lookup_exact,
     "exact": lookup_exact,
     "iexact": lookup_iexact,
@@ -306,16 +313,16 @@ LOOKUP_NAME_MAP: 'Mapping[str, "LookupProtocol"]' = {
 
 
 class PKRequiredException(Exception):
-    def __init__(self, *args: object):
+    def __init__(self, *args: object) -> None:
         return super().__init__("items() require a pk_key exists")
 
 
 class OpNotFound(ValueError):
-    def __init__(self, op: str, *args: object):
+    def __init__(self, op: str, *args: object) -> None:
         return super().__init__(f"{op} not in LOOKUP_NAME_MAP")
 
 
-class QueryList(t.Generic[T], t.List[T]):
+class QueryList(list[T], t.Generic[T]):
     """Filter list of object/dictionaries. For small, local datasets.
 
     *Experimental, unstable*.
@@ -464,15 +471,15 @@ class QueryList(t.Generic[T], t.List[T]):
     []
     """
 
-    data: "Sequence[T]"
-    pk_key: t.Optional[str]
+    data: Sequence[T]
+    pk_key: str | None
 
-    def __init__(self, items: t.Optional["Iterable[T]"] = None) -> None:
+    def __init__(self, items: Iterable[T] | None = None) -> None:
         super().__init__(items if items is not None else [])
 
-    def items(self) -> t.List[t.Tuple[str, T]]:
+    def items(self) -> list[tuple[str, T]]:
         if self.pk_key is None:
-            raise PKRequiredException()
+            raise PKRequiredException
         return [(getattr(item, self.pk_key), item) for item in self]
 
     def __eq__(
@@ -492,18 +499,17 @@ class QueryList(t.Generic[T], t.List[T]):
                         for key in a_keys:
                             if abs(a[key] - b[key]) > 1:
                                 return False
-                else:
-                    if a != b:
-                        return False
+                elif a != b:
+                    return False
 
             return True
         return False
 
     def filter(
         self,
-        matcher: t.Optional[t.Union[t.Callable[[T], bool], T]] = None,
+        matcher: Callable[[T], bool] | T | None = None,
         **kwargs: t.Any,
-    ) -> "QueryList[T]":
+    ) -> QueryList[T]:
         """Filter list of objects."""
 
         def filter_lookup(obj: t.Any) -> bool:
@@ -527,27 +533,26 @@ class QueryList(t.Generic[T], t.List[T]):
             return True
 
         if callable(matcher):
-            _filter = matcher
+            filter_ = matcher
         elif matcher is not None:
 
-            def val_match(obj: t.Union[str, t.List[t.Any], T]) -> bool:
+            def val_match(obj: str | list[t.Any] | T) -> bool:
                 if isinstance(matcher, list):
                     return obj in matcher
-                else:
-                    return bool(obj == matcher)
+                return bool(obj == matcher)
 
-            _filter = val_match
+            filter_ = val_match
         else:
-            _filter = filter_lookup
+            filter_ = filter_lookup
 
-        return self.__class__(k for k in self if _filter(k))
+        return self.__class__(k for k in self if filter_(k))
 
     def get(
         self,
-        matcher: t.Optional[t.Union[t.Callable[[T], bool], T]] = None,
-        default: t.Optional[t.Any] = no_arg,
+        matcher: Callable[[T], bool] | T | None = None,
+        default: t.Any | None = no_arg,
         **kwargs: t.Any,
-    ) -> t.Optional[T]:
+    ) -> T | None:
         """Retrieve one object.
 
         Raises :exc:`MultipleObjectsReturned` if multiple objects found.
@@ -556,9 +561,9 @@ class QueryList(t.Generic[T], t.List[T]):
         """
         objs = self.filter(matcher=matcher, **kwargs)
         if len(objs) > 1:
-            raise MultipleObjectsReturned()
-        elif len(objs) == 0:
+            raise MultipleObjectsReturned
+        if len(objs) == 0:
             if default == no_arg:
-                raise ObjectDoesNotExist()
+                raise ObjectDoesNotExist
             return default
         return objs[0]
