@@ -33,7 +33,13 @@ from .common import (
 )
 
 if t.TYPE_CHECKING:
-    from typing_extensions import TypeAlias
+    import sys
+    import types
+
+    if sys.version_info >= (3, 10):
+        from typing import Self, TypeAlias
+    else:
+        from typing_extensions import Self, TypeAlias
 
     DashLiteral: TypeAlias = t.Literal["-"]
 
@@ -59,6 +65,8 @@ class Server(EnvironmentMixin):
     socket_path : str, optional
     config_file : str, optional
     colors : str, optional
+    on_init : callable, optional
+    socket_name_factory : callable, optional
 
     Examples
     --------
@@ -77,6 +85,13 @@ class Server(EnvironmentMixin):
     >>> server.sessions[0].active_pane
     Pane(%1 Window(@1 1:..., Session($1 ...)))
 
+    The server can be used as a context manager to ensure proper cleanup:
+
+    >>> with Server() as server:
+    ...     session = server.new_session()
+    ...     # Do work with the session
+    ...     # Server will be killed automatically when exiting the context
+
     References
     ----------
     .. [server_manual] CLIENTS AND SESSIONS. openbsd manpage for TMUX(1)
@@ -110,6 +125,8 @@ class Server(EnvironmentMixin):
         socket_path: str | pathlib.Path | None = None,
         config_file: str | None = None,
         colors: int | None = None,
+        on_init: t.Callable[[Server], None] | None = None,
+        socket_name_factory: t.Callable[[], str] | None = None,
         **kwargs: t.Any,
     ) -> None:
         EnvironmentMixin.__init__(self, "-g")
@@ -120,6 +137,8 @@ class Server(EnvironmentMixin):
             self.socket_path = socket_path
         elif socket_name is not None:
             self.socket_name = socket_name
+        elif socket_name_factory is not None:
+            self.socket_name = socket_name_factory()
 
         tmux_tmpdir = pathlib.Path(os.getenv("TMUX_TMPDIR", "/tmp"))
         socket_name = self.socket_name or "default"
@@ -137,6 +156,39 @@ class Server(EnvironmentMixin):
         if colors:
             self.colors = colors
 
+        if on_init is not None:
+            on_init(self)
+
+    def __enter__(self) -> Self:
+        """Enter the context, returning self.
+
+        Returns
+        -------
+        :class:`Server`
+            The server instance
+        """
+        return self
+
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        exc_tb: types.TracebackType | None,
+    ) -> None:
+        """Exit the context, killing the server if it exists.
+
+        Parameters
+        ----------
+        exc_type : type[BaseException] | None
+            The type of the exception that was raised
+        exc_value : BaseException | None
+            The instance of the exception that was raised
+        exc_tb : types.TracebackType | None
+            The traceback of the exception that was raised
+        """
+        if self.is_alive():
+            self.kill()
+
     def is_alive(self) -> bool:
         """Return True if tmux server alive.
 
