@@ -1,5 +1,7 @@
 """Tests for libtmux object model, querying and traversal, etc."""
-import pathlib
+
+from __future__ import annotations
+
 import typing as t
 
 import pytest
@@ -11,17 +13,20 @@ from libtmux._internal.query_list import (
 )
 from libtmux.constants import ResizeAdjustmentDirection
 from libtmux.pane import Pane
-from libtmux.server import Server
 from libtmux.session import Session
 from libtmux.window import Window
 
 if t.TYPE_CHECKING:
+    import pathlib
+
+    from libtmux.server import Server
+
     ListCmd = t.Literal["list-sessions", "list-windows", "list-panes"]
-    ListExtraArgs = t.Optional[t.Tuple[str]]
+    ListExtraArgs = t.Optional[tuple[str]]
 
 
-OutputRaw = t.Dict[str, t.Any]
-OutputsRaw = t.List[OutputRaw]
+OutputRaw = dict[str, t.Any]
+OutputsRaw = list[OutputRaw]
 
 
 def test_pane(
@@ -33,23 +38,23 @@ def test_pane(
     monkeypatch.chdir(tmp_path)
 
     try:
-        __session = server.sessions[0]
+        session_ = server.sessions[0]
     except Exception:
-        __session = server.new_session()
+        session_ = server.new_session()
 
-    assert __session is not None
+    assert session_ is not None
 
-    __window = __session.attached_window
-    __window.split_window()
-    __pane = __window.split_window()
-    __window.select_layout("main-vertical")
+    window_ = session_.active_window
+    window_.split()
+    pane_ = window_.split()
+    window_.select_layout("main-vertical")
 
-    assert __pane is not None
-    assert __pane.pane_id is not None
+    assert pane_ is not None
+    assert pane_.pane_id is not None
 
-    assert isinstance(__pane.pane_id, str)
+    assert isinstance(pane_.pane_id, str)
 
-    pane = Pane.from_pane_id(server=__pane.server, pane_id=__pane.pane_id)
+    pane = Pane.from_pane_id(server=pane_.server, pane_id=pane_.pane_id)
 
     assert isinstance(pane, Pane)
 
@@ -62,7 +67,8 @@ def test_pane(
 
     pane.resize_pane(adjustment_direction=ResizeAdjustmentDirection.Down, adjustment=25)
     pane.resize_pane(
-        adjustment_direction=ResizeAdjustmentDirection.Right, adjustment=25
+        adjustment_direction=ResizeAdjustmentDirection.Right,
+        adjustment=25,
     )
 
     assert old_pane_size != pane.pane_height
@@ -124,12 +130,12 @@ def test_pane(
     #
 
     # Window-level
-    new_pane = window.split_window()
+    new_pane = window.split()
     assert new_pane.pane_id != pane.pane_id
     assert new_pane.window_id == window.window_id
 
     # Pane-level
-    new_pane_2 = new_pane.split_window()
+    new_pane_2 = new_pane.split()
     assert new_pane_2.pane_id != new_pane.pane_id
     assert new_pane_2.window_id == new_pane.window_id
 
