@@ -1,19 +1,26 @@
 """Tests for libtmux Pane object."""
+
+from __future__ import annotations
+
 import logging
 import shutil
+import typing as t
 
 import pytest
 
-from libtmux.common import has_gte_version, has_lt_version
-from libtmux.constants import ResizeAdjustmentDirection
-from libtmux.session import Session
+from libtmux.common import has_gte_version, has_lt_version, has_lte_version
+from libtmux.constants import PaneDirection, ResizeAdjustmentDirection
+from libtmux.test.retry import retry_until
+
+if t.TYPE_CHECKING:
+    from libtmux.session import Session
 
 logger = logging.getLogger(__name__)
 
 
 def test_send_keys(session: Session) -> None:
     """Verify Pane.send_keys()."""
-    pane = session.attached_window.attached_pane
+    pane = session.active_window.active_pane
     assert pane is not None
     pane.send_keys("c-c", literal=True)
 
@@ -27,8 +34,8 @@ def test_send_keys(session: Session) -> None:
 def test_set_height(session: Session) -> None:
     """Verify Pane.set_height()."""
     window = session.new_window(window_name="test_set_height")
-    window.split_window()
-    pane1 = window.attached_pane
+    window.split()
+    pane1 = window.active_pane
     assert pane1 is not None
     pane1_height = pane1.pane_height
 
@@ -41,10 +48,10 @@ def test_set_height(session: Session) -> None:
 def test_set_width(session: Session) -> None:
     """Verify Pane.set_width()."""
     window = session.new_window(window_name="test_set_width")
-    window.split_window()
+    window.split()
 
     window.select_layout("main-vertical")
-    pane1 = window.attached_pane
+    pane1 = window.active_pane
     assert pane1 is not None
     pane1_width = pane1.pane_width
 
@@ -66,7 +73,7 @@ def test_capture_pane(session: Session) -> None:
         window_name="capture_pane",
         window_shell=f"{env} PS1='$ ' sh",
     )
-    pane = session.attached_window.attached_pane
+    pane = session.active_window.active_pane
     assert pane is not None
     pane_contents = "\n".join(pane.capture_pane())
     assert pane_contents == "$"
@@ -91,7 +98,7 @@ def test_capture_pane_start(session: Session) -> None:
         window_name="capture_pane_start",
         window_shell=f"{env} PS1='$ ' sh",
     )
-    pane = session.attached_window.attached_pane
+    pane = session.active_window.active_pane
     assert pane is not None
     pane_contents = "\n".join(pane.capture_pane())
     assert pane_contents == "$"
@@ -99,14 +106,31 @@ def test_capture_pane_start(session: Session) -> None:
     pane_contents = "\n".join(pane.capture_pane())
     assert pane_contents == '$ printf "%s"\n$'
     pane.send_keys("clear -x", literal=True, suppress_history=False)
-    pane_contents = "\n".join(pane.capture_pane())
-    assert pane_contents == "$"
-    pane_contents_start = pane.capture_pane(start=-2)
-    assert pane_contents_start[0] == '$ printf "%s"'
-    assert pane_contents_start[1] == "$ clear -x"
-    assert pane_contents_start[-1] == "$"
-    pane_contents_start = pane.capture_pane(start="-")
-    assert pane_contents == "$"
+
+    def wait_until_pane_cleared() -> bool:
+        pane_contents = "\n".join(pane.capture_pane())
+        return "clear -x" not in pane_contents
+
+    retry_until(wait_until_pane_cleared, 1, raises=True)
+
+    def pane_contents_shell_prompt() -> bool:
+        pane_contents = "\n".join(pane.capture_pane())
+        return pane_contents == "$"
+
+    retry_until(pane_contents_shell_prompt, 1, raises=True)
+
+    pane_contents_history_start = pane.capture_pane(start=-2)
+    assert pane_contents_history_start[0] == '$ printf "%s"'
+    assert pane_contents_history_start[1] == "$ clear -x"
+    assert pane_contents_history_start[-1] == "$"
+
+    pane.send_keys("")
+
+    def pane_contents_capture_visible_only_shows_prompt() -> bool:
+        pane_contents = "\n".join(pane.capture_pane(start=1))
+        return pane_contents == "$"
+
+    assert retry_until(pane_contents_capture_visible_only_shows_prompt, 1, raises=True)
 
 
 def test_capture_pane_end(session: Session) -> None:
@@ -119,7 +143,7 @@ def test_capture_pane_end(session: Session) -> None:
         window_name="capture_pane_end",
         window_shell=f"{env} PS1='$ ' sh",
     )
-    pane = session.attached_window.attached_pane
+    pane = session.active_window.active_pane
     assert pane is not None
     pane_contents = "\n".join(pane.capture_pane())
     assert pane_contents == "$"
@@ -132,6 +156,36 @@ def test_capture_pane_end(session: Session) -> None:
     assert pane_contents == '$ printf "%s"\n$'
 
 
+@pytest.mark.skipif(
+    has_lte_version("3.1"),
+    reason="3.2 has the -Z flag on split-window",
+)
+def test_pane_split_window_zoom(
+    session: Session,
+) -> None:
+    """Verify splitting window with zoom."""
+    window_without_zoom = session.new_window(window_name="split_without_zoom")
+    initial_pane_without_zoom = window_without_zoom.active_pane
+    assert initial_pane_without_zoom is not None
+    window_with_zoom = session.new_window(window_name="split_with_zoom")
+    initial_pane_with_zoom = window_with_zoom.active_pane
+    assert initial_pane_with_zoom is not None
+    pane_without_zoom = initial_pane_without_zoom.split(
+        zoom=False,
+    )
+    pane_with_zoom = initial_pane_with_zoom.split(
+        zoom=True,
+    )
+
+    assert pane_without_zoom.width == pane_without_zoom.window_width
+    assert pane_without_zoom.height is not None
+    assert pane_without_zoom.window_height is not None
+    assert pane_without_zoom.height < pane_without_zoom.window_height
+
+    assert pane_with_zoom.width == pane_with_zoom.window_width
+    assert pane_with_zoom.height == pane_with_zoom.window_height
+
+
 @pytest.mark.skipif(
     has_lt_version("2.9"),
     reason="resize-window only exists in tmux 2.9+",
@@ -142,9 +196,9 @@ def test_resize_pane(
     """Verify resizing window."""
     session.cmd("detach-client", "-s")
 
-    window = session.attached_window
-    pane = window.split_window(attach=False)
-    window.split_window(vertical=True, attach=False)
+    window = session.active_window
+    pane = window.split(attach=False)
+    window.split(direction=PaneDirection.Above, attach=False)
 
     assert pane is not None
 
@@ -222,3 +276,62 @@ def test_resize_pane(
     )
     pane_height_expanded = int(pane.pane_height)
     assert pane_height_before < pane_height_expanded
+
+
+def test_split_pane_size(session: Session) -> None:
+    """Pane.split()."""
+    window = session.new_window(window_name="split window size")
+    window.resize(height=100, width=100)
+    pane = window.active_pane
+    assert pane is not None
+
+    if has_gte_version("3.1"):
+        short_pane = pane.split(size=10)
+        assert short_pane.pane_height == "10"
+
+        assert short_pane.at_left
+        assert short_pane.at_right
+        assert not short_pane.at_top
+        assert short_pane.at_bottom
+
+        narrow_pane = pane.split(direction=PaneDirection.Right, size=10)
+        assert narrow_pane.pane_width == "10"
+
+        assert not narrow_pane.at_left
+        assert narrow_pane.at_right
+        assert narrow_pane.at_top
+        assert not narrow_pane.at_bottom
+
+        new_pane = pane.split(size="10%")
+        assert new_pane.pane_height == "8"
+
+        new_pane = short_pane.split(direction=PaneDirection.Right, size="10%")
+        assert new_pane.pane_width == "10"
+
+        assert not new_pane.at_left
+        assert new_pane.at_right
+    else:
+        window_height_before = (
+            int(window.window_height) if isinstance(window.window_height, str) else 0
+        )
+        window_width_before = (
+            int(window.window_width) if isinstance(window.window_width, str) else 0
+        )
+        new_pane = pane.split(size="10%")
+        assert new_pane.pane_height == str(int(window_height_before * 0.1))
+
+        new_pane = new_pane.split(direction=PaneDirection.Right, size="10%")
+        assert new_pane.pane_width == str(int(window_width_before * 0.1))
+
+
+def test_pane_context_manager(session: Session) -> None:
+    """Test Pane context manager functionality."""
+    window = session.new_window()
+    with window.split() as pane:
+        pane.send_keys('echo "Hello"')
+        assert pane in window.panes
+        assert len(window.panes) == 2  # Initial pane + new pane
+
+    # Pane should be killed after exiting context
+    assert pane not in window.panes
+    assert len(window.panes) == 1  # Only initial pane remains
